@@ -8,7 +8,8 @@ from simple_salesforce.api import (
     SalesforceExpiredSession,
     SalesforceRefusedRequest,
     SalesforceResourceNotFound,
-    SalesforceGeneralError
+    SalesforceGeneralError,
+    SalesforceMalformedRequest
 )
 
 from simple_salesforce.login import (
