@@ -58,7 +58,7 @@ class Salesforce(object):
         self.proxies = kwargs.get('proxies')
 
         # Determine if the user wants to use our username/password auth or pass in their own information
-        if ('username' in kwargs) and ('password' in kwargs) and ('security_token' in kwargs):
+        if 'username' in kwargs and 'password' in kwargs and 'security_token' in kwargs:
             self.auth_type = "password"
             username = kwargs['username']
             password = kwargs['password']
@@ -73,18 +73,18 @@ class Salesforce(object):
                 sf_version=self.sf_version,
                 proxies=self.proxies)
 
-        elif ('session_id' in kwargs) and (('instance' in kwargs) or ('instance_url' in kwargs)):
+        elif 'session_id' in kwargs and ('instance' in kwargs or 'instance_url' in kwargs):
             self.auth_type = "direct"
             self.session_id = kwargs['session_id']
 
             # If the user provides the full url (as returned by the OAuth interface for
             # example) extract the hostname (which we rely on)
-            if ('instance_url' in kwargs):
+            if 'instance_url' in kwargs:
                 self.sf_instance = urlparse(kwargs['instance_url']).hostname
             else:
                 self.sf_instance = kwargs['instance']
 
-        elif ('username' in kwargs) and ('password' in kwargs) and ('organizationId' in kwargs):
+        elif 'username' in kwargs and 'password' in kwargs and 'organizationId' in kwargs:
             self.auth_type = 'ipfilter'
             username = kwargs['username']
             password = kwargs['password']
@@ -115,9 +115,12 @@ class Salesforce(object):
             'Authorization': 'Bearer ' + self.session_id,
             'X-PrettyPrint': '1'
         }
+
         self.base_url = ('https://{instance}/services/data/v{version}/'
                          .format(instance=self.sf_instance,
                                  version=self.sf_version))
+        self.apex_url = ('https://{instance}/services/apexrest/'
+                         .format(instance=self.sf_instance))
 
     def describe(self):
         url = self.base_url + "sobjects"
@@ -146,6 +149,60 @@ class Salesforce(object):
         """
         return SFType(name, self.session_id, self.sf_instance, self.sf_version, self.proxies)
 
+    # User utlity methods
+    def set_password(self, user, password):
+        """Sets the password of a user
+
+        salesforce dev documentation link:
+        https://www.salesforce.com/us/developer/docs/api_rest/Content/dome_sobject_user_password.htm
+
+        Arguments:
+
+        * user: the userID of the user to set
+        * password: the new password
+        """
+
+        url = self.base_url + 'sobjects/User/%s/password' % user
+        params = { 'NewPassword' : password, }
+
+        result = self.request.post(url, headers=self.headers, data=json.dumps(params))
+
+        # salesforce return 204 No Content when the request is successful
+        if result.status_code != 200 or result.status_code != 204:
+            raise SalesforceGeneralError(result.content)
+        json_result = result.json(object_pairs_hook=OrderedDict)
+        if len(json_result) == 0:
+            return None
+        else:
+            return json_result
+
+    def setPassword(self, user, password):
+        import warnings
+        warnings.warn(
+            "This method has been deprecated. Please use set_password instread.", DeprecationWarning)
+        return self.set_password(user, password)
+
+    # Generic Rest Function
+    def restful(self,path,params):
+        """Allows you to make a direct REST call if you know the path
+
+        Arguments:
+
+        * path: The path of the request
+            Example: sobjects/User/ABC123/password'
+        * params: dict of parameters to pass to the path
+        """
+
+        url = self.base_url + path
+        result = self.request.get(url, headers=self.headers, params=params)
+        if result.status_code != 200:
+            raise SalesforceGeneralError(result.content)
+        json_result = result.json(object_pairs_hook=OrderedDict)
+        if len(json_result) == 0:
+            return None
+        else:
+            return json_result
+        
     # Search Functions
     def search(self, search):
         """Returns the result of a Salesforce search as a dict decoded from
@@ -179,11 +236,11 @@ class Salesforce(object):
                     string will be wrapped to read `FIND {Waldo}` before being
                     sent to Salesforce
         """
-        search_string = 'FIND {{{search_string}}}'.format(search_string=search)
+        search_string = u'FIND {{{search_string}}}'.format(search_string=search)
         return self.search(search_string)
 
     # Query Handler
-    def query(self, query):
+    def query(self, query, **kwargs):
         """Return the result of a Salesforce SOQL query as a dict decoded from
         the Salesforce response JSON payload.
 
@@ -195,14 +252,14 @@ class Salesforce(object):
         url = self.base_url + 'query/'
         params = {'q': query}
         # `requests` will correctly encode the query string passed as `params`
-        result = self.request.get(url, headers=self.headers, params=params)
+        result = self.request.get(url, headers=self.headers, params=params, **kwargs)
 
         if result.status_code != 200:
             _exception_handler(result)
 
         return result.json(object_pairs_hook=OrderedDict)
 
-    def query_more(self, next_records_identifier, identifier_is_url=False):
+    def query_more(self, next_records_identifier, identifier_is_url=False, **kwargs):
         """Retrieves more results from a query that returned more results
         than the batch maximum. Returns a dict decoded from the Salesforce
         response JSON payload.
@@ -219,20 +276,20 @@ class Salesforce(object):
         """
         if identifier_is_url:
             # Don't use `self.base_url` here because the full URI is provided
-            url = ('https://{instance}{next_record_url}'
+            url = (u'https://{instance}{next_record_url}'
                    .format(instance=self.sf_instance,
                            next_record_url=next_records_identifier))
         else:
             url = self.base_url + 'query/{next_record_id}'
             url = url.format(next_record_id=next_records_identifier)
-        result = self.request.get(url, headers=self.headers)
+        result = self.request.get(url, headers=self.headers, **kwargs)
 
         if result.status_code != 200:
             _exception_handler(result)
 
         return result.json(object_pairs_hook=OrderedDict)
 
-    def query_all(self, query):
+    def query_all(self, query, **kwargs):
         """Returns the full set of results for the `query`. This is a
         convenience wrapper around `query(...)` and `query_more(...)`.
 
@@ -246,7 +303,7 @@ class Salesforce(object):
         * query -- the SOQL query to send to Salesforce, e.g.
                    `SELECT Id FROM Lead WHERE Email = "waldo@somewhere.com"`
         """
-        def get_all_results(previous_result):
+        def get_all_results(previous_result, **kwargs):
             """Inner function for recursing until there are no more results.
 
             Returns the full set of results that will be the return value for
@@ -261,19 +318,46 @@ class Salesforce(object):
                 return previous_result
             else:
                 result = self.query_more(previous_result['nextRecordsUrl'],
-                                         identifier_is_url=True)
+                                         identifier_is_url=True, **kwargs)
                 result['totalSize'] += previous_result['totalSize']
                 # Include the new list of records with the previous list
                 previous_result['records'].extend(result['records'])
                 result['records'] = previous_result['records']
                 # Continue the recursion
-                return get_all_results(result)
+                return get_all_results(result, **kwargs)
 
         # Make the initial query to Salesforce
-        result = self.query(query)
+        result = self.query(query, **kwargs)
         # The number of results might have exceeded the Salesforce batch limit
         # so check whether there are more results and retrieve them if so.
-        return get_all_results(result)
+        return get_all_results(result, **kwargs)
+
+    def apexecute(self, action, method='GET', data=None):
+        result = self._call_salesforce(method, self.apex_url + action, data=json.dumps(data))
+
+        if result.status_code == 200:
+            try:
+                response_content = result.json()
+            except Exception:
+                response_content = result.text
+            return response_content
+
+    def _call_salesforce(self, method, url, **kwargs):
+        """Utility method for performing HTTP call to Salesforce.
+
+        Returns a `requests.result` object.
+        """
+        headers = {
+            'Content-Type': 'application/json',
+            'Authorization': 'Bearer ' + self.session_id,
+            'X-PrettyPrint': '1'
+        }
+        result = self.request.request(method, url, headers=headers, **kwargs)
+
+        if result.status_code >= 300:
+            _exception_handler(result)
+
+        return result
 
 
 class SFType(object):
@@ -293,12 +377,13 @@ class SFType(object):
         """
         self.session_id = session_id
         self.name = object_name
-        self.base_url = ('https://{instance}/services/data/v{sf_version}/sobjects/{object_name}/'
+        self.request = requests.Session()
+        self.request.proxies = proxies
+
+        self.base_url = (u'https://{instance}/services/data/v{sf_version}/sobjects/{object_name}/'
                          .format(instance=sf_instance,
                                  object_name=object_name,
                                  sf_version=sf_version))
-        self.request = requests.Session()
-        self.request.proxies = proxies
 
     def metadata(self):
         """Returns the result of a GET to `.../{object_name}/` as a dict
@@ -380,7 +465,7 @@ class SFType(object):
         return result.status_code
 
     def delete(self, record_id):
-        """Deletess an SObject using a DELETE to
+        """Deletes an SObject using a DELETE to
         `.../{object_name}/{record_id}`.
 
         Returns a dict decoded from the JSON payload returned by Salesforce.
@@ -397,7 +482,7 @@ class SFType(object):
          .../deleted/?start=2013-05-05T00:00:00+00:00&end=2013-05-10T00:00:00+00:00
 
         * start -- start datetime object
-        * end -- end dattime object
+        * end -- end datetime object
         """
         url = self.base_url + 'deleted/?start={start}&end={end}'.format(
             start=date_to_iso8601(start), end=date_to_iso8601(end))
@@ -411,7 +496,7 @@ class SFType(object):
          .../updated/?start=2014-03-20T00:00:00+00:00&end=2014-03-22T00:00:00+00:00
 
         * start -- start datetime object
-        * end -- end dattime object
+        * end -- end datetime object
         """
         url = self.base_url + 'updated/?start={start}&end={end}'.format(
             start=date_to_iso8601(start), end=date_to_iso8601(end))
@@ -475,27 +560,27 @@ def _exception_handler(result, name=""):
         response_content = result.text
 
     if result.status_code == 300:
-        message = "More than one record for {url}. Response content: {content}"
+        message = u"More than one record for {url}. Response content: {content}"
         message = message.format(url=url, content=response_content)
         raise SalesforceMoreThanOneRecord(message)
     elif result.status_code == 400:
-        message = "Malformed request {url}. Response content: {content}"
+        message = u"Malformed request {url}. Response content: {content}"
         message = message.format(url=url, content=response_content)
         raise SalesforceMalformedRequest(message)
     elif result.status_code == 401:
-        message = "Expired session for {url}. Response content: {content}"
+        message = u"Expired session for {url}. Response content: {content}"
         message = message.format(url=url, content=response_content)
         raise SalesforceExpiredSession(message)
     elif result.status_code == 403:
-        message = "Request refused for {url}. Response content: {content}"
+        message = u"Request refused for {url}. Response content: {content}"
         message = message.format(url=url, content=response_content)
         raise SalesforceRefusedRequest(message)
     elif result.status_code == 404:
-        message = 'Resource {name} Not Found. Response content: {content}'
+        message = u'Resource {name} Not Found. Response content: {content}'
         message = message.format(name=name, content=response_content)
         raise SalesforceResourceNotFound(message)
     else:
-        message = 'Error Code {status}. Response content: {content}'
+        message = u'Error Code {status}. Response content: {content}'
         message = message.format(status=result.status_code, content=response_content)
         raise SalesforceGeneralError(message)
 
